@@ -16,6 +16,7 @@ import re
 import unittest
 
 from oslo_config import types
+from six.moves import range as compat_range
 
 
 class ConfigTypeTests(unittest.TestCase):
@@ -62,6 +63,15 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         self.type_instance = types.String(choices=['foo', 'bar'])
         self.assertConvertedValue('foo', 'foo')
 
+    def test_listed_value_tuple(self):
+        self.type_instance = types.String(choices=('foo', 'bar'))
+        self.assertConvertedValue('foo', 'foo')
+
+    def test_listed_value_dict(self):
+        self.type_instance = types.String(choices=[
+            ('foo', 'ab'), ('bar', 'xy')])
+        self.assertConvertedValue('foo', 'foo')
+
     def test_unlisted_value(self):
         self.type_instance = types.String(choices=['foo', 'bar'])
         self.assertInvalid('baz')
@@ -91,6 +101,14 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.String(choices=['foo', 'bar'])
         self.assertEqual('String(choices=[\'foo\', \'bar\'])', repr(t))
 
+    def test_repr_with_choices_tuple(self):
+        t = types.String(choices=('foo', 'bar'))
+        self.assertEqual('String(choices=[\'foo\', \'bar\'])', repr(t))
+
+    def test_repr_with_choices_dict(self):
+        t = types.String(choices=[('foo', 'ab'), ('bar', 'xy')])
+        self.assertEqual('String(choices=[\'foo\', \'bar\'])', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.String() == types.String())
 
@@ -99,14 +117,15 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         t2 = types.String(choices=['foo', 'bar'])
         t3 = types.String(choices=('foo', 'bar'))
         t4 = types.String(choices=['bar', 'foo'])
-        self.assertTrue(t1 == t2)
-        self.assertTrue(t1 == t3)
-        self.assertTrue(t1 == t4)
+        t5 = types.String(choices=[('foo', 'ab'), ('bar', 'xy')])
+        self.assertTrue(t1 == t2 == t3 == t4 == t5)
 
     def test_not_equal_with_different_choices(self):
         t1 = types.String(choices=['foo', 'bar'])
         t2 = types.String(choices=['foo', 'baz'])
+        t3 = types.String(choices=('foo', 'baz'))
         self.assertFalse(t1 == t2)
+        self.assertFalse(t1 == t3)
 
     def test_equal_with_equal_quote_falgs(self):
         t1 = types.String(quotes=True)
@@ -269,6 +288,14 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.Integer(choices=[80, 457])
         self.assertEqual('Integer(choices=[80, 457])', repr(t))
 
+    def test_repr_with_choices_tuple(self):
+        t = types.Integer(choices=(80, 457))
+        self.assertEqual('Integer(choices=[80, 457])', repr(t))
+
+    def test_repr_with_choices_dict(self):
+        t = types.Integer(choices=[(80, 'ab'), (457, 'xy')])
+        self.assertEqual('Integer(choices=[80, 457])', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.Integer() == types.Integer())
 
@@ -286,7 +313,9 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
     def test_equal_with_same_choices(self):
         t1 = types.Integer(choices=[80, 457])
         t2 = types.Integer(choices=[457, 80])
-        self.assertTrue(t1 == t2)
+        t3 = types.Integer(choices=(457, 80))
+        t4 = types.Integer(choices=[(80, 'ab'), (457, 'xy')])
+        self.assertTrue(t1 == t2 == t3 == t4)
 
     def test_not_equal(self):
         self.assertFalse(types.Integer(min=123) == types.Integer(min=456))
@@ -352,21 +381,24 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertRaises(ValueError, t, 201)
         self.assertRaises(ValueError, t, -457)
 
-    def test_with_choices_list(self):
-        t = types.Integer(choices=[80, 457])
+    def _test_with_choices(self, t):
         self.assertRaises(ValueError, t, 1)
         self.assertRaises(ValueError, t, 200)
         self.assertRaises(ValueError, t, -457)
         t(80)
         t(457)
 
+    def test_with_choices_list(self):
+        t = types.Integer(choices=[80, 457])
+        self._test_with_choices(t)
+
     def test_with_choices_tuple(self):
         t = types.Integer(choices=(80, 457))
-        self.assertRaises(ValueError, t, 1)
-        self.assertRaises(ValueError, t, 200)
-        self.assertRaises(ValueError, t, -457)
-        t(80)
-        t(457)
+        self._test_with_choices(t)
+
+    def test_with_choices_dict(self):
+        t = types.Integer(choices=[(80, 'ab'), (457, 'xy')])
+        self._test_with_choices(t)
 
 
 class FloatTypeTests(TypeTestHelper, unittest.TestCase):
@@ -486,6 +518,10 @@ class ListTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertConvertedValue(' foo bar ',
                                   ['foo bar'])
 
+    def test_tuple_of_values(self):
+        self.assertConvertedValue(('foo', 'bar'),
+                                  ['foo', 'bar'])
+
     def test_list_of_values(self):
         self.assertConvertedValue(' foo bar, baz ',
                                   ['foo bar',
@@ -498,6 +534,10 @@ class ListTypeTests(TypeTestHelper, unittest.TestCase):
                                    'bar, baz',
                                    'bam'])
 
+    def test_list_of_values_containing_trailing_comma(self):
+        self.assertConvertedValue('foo, bar, baz, ',
+                                  ['foo', 'bar', 'baz'])
+
     def test_list_of_lists(self):
         self.type_instance = types.List(
             types.List(types.String(), bounds=True)
@@ -510,6 +550,16 @@ class ListTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertConvertedValue('1,2,3,5',
                                   [1, 2, 3, 5])
 
+    def test_list_of_custom_type_containing_trailing_comma(self):
+        self.type_instance = types.List(types.Integer())
+        self.assertConvertedValue('1,2,3,5,',
+                                  [1, 2, 3, 5])
+
+    def test_tuple_of_custom_type(self):
+        self.type_instance = types.List(types.Integer())
+        self.assertConvertedValue(('1', '2', '3', '5'),
+                                  [1, 2, 3, 5])
+
     def test_bounds_parsing(self):
         self.type_instance = types.List(types.Integer(), bounds=True)
         self.assertConvertedValue('[1,2,3]', [1, 2, 3])
@@ -542,6 +592,45 @@ class ListTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertFalse(types.List() == types.Integer())
 
 
+class RangeTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.Range()
+
+    def assertRange(self, s, r1, r2, step=1):
+        self.assertEqual(list(compat_range(r1, r2, step)),
+                         list(self.type_instance(s)))
+
+    def test_range(self):
+        self.assertRange('0-2', 0, 3)
+        self.assertRange('-2-0', -2, 1)
+        self.assertRange('2-0', 2, -1, -1)
+        self.assertRange('-3--1', -3, 0)
+        self.assertRange('-1--3', -1, -4, -1)
+        self.assertRange('-1', -1, 0)
+        self.assertInvalid('--1')
+        self.assertInvalid('4-')
+        self.assertInvalid('--')
+        self.assertInvalid('1.1-1.2')
+        self.assertInvalid('a-b')
+
+    def test_range_bounds(self):
+        self.type_instance = types.Range(1, 3)
+        self.assertRange('1-3', 1, 4)
+        self.assertRange('2-2', 2, 3)
+        self.assertRange('2', 2, 3)
+        self.assertInvalid('1-4')
+        self.assertInvalid('0-3')
+        self.assertInvalid('0-4')
+
+    def test_range_exclusive(self):
+        self.type_instance = types.Range(inclusive=False)
+        self.assertRange('0-2', 0, 2)
+        self.assertRange('-2-0', -2, 0)
+        self.assertRange('2-0', 2, 0, -1)
+        self.assertRange('-3--1', -3, -1)
+        self.assertRange('-1--3', -1, -3, -1)
+        self.assertRange('-1', -1, -1)
+
+
 class DictTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Dict()
 
@@ -647,6 +736,32 @@ class IPv6AddressTypeTests(IPAddressTypeTests):
         self.assertInvalid('192.168.0.1')
 
 
+class HostAddressTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.HostAddress()
+
+    def test_invalid_host_addresses(self):
+        self.assertInvalid('-1')
+        self.assertInvalid('_foo')
+        self.assertInvalid('3.14')
+        self.assertInvalid('10.0')
+        self.assertInvalid('host..name')
+        self.assertInvalid('org.10')
+        self.assertInvalid('0.0.00')
+
+    def test_valid_host_addresses(self):
+        self.assertConvertedValue('foo.bar', 'foo.bar')
+        self.assertConvertedValue('192.168.0.1', '192.168.0.1')
+        self.assertConvertedValue('abcd:ef::1', 'abcd:ef::1')
+        self.assertConvertedValue('home-site-here.org.com',
+                                  'home-site-here.org.com')
+        self.assertConvertedValue('3com.com', '3com.com')
+        self.assertConvertedValue('10.org', '10.org')
+        self.assertConvertedValue('cell1.nova.site1', 'cell1.nova.site1')
+        self.assertConvertedValue('ab-c.com', 'ab-c.com')
+        self.assertConvertedValue('abc.com-org', 'abc.com-org')
+        self.assertConvertedValue('abc.0-0', 'abc.0-0')
+
+
 class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Hostname()
 
@@ -686,6 +801,12 @@ class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertInvalid(".host.name.com")
         self.assertInvalid("no spaces")
 
+    def test_invalid_hostnames_with_numeric_characters(self):
+        self.assertInvalid("10.0.0.0")
+        self.assertInvalid("3.14")
+        self.assertInvalid("org.10")
+        self.assertInvalid('0.0.00')
+
     def test_no_start_end_hyphens(self):
         self.assertInvalid("-host.com")
         self.assertInvalid("-hostname.com-")
@@ -699,9 +820,13 @@ class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertConvertedEqual('cell1.nova.site1')
         self.assertConvertedEqual('site01001')
         self.assertConvertedEqual('home-site-here.org.com')
-        self.assertConvertedEqual('192.168.0.1')
-        self.assertConvertedEqual('1.1.1')
         self.assertConvertedEqual('localhost')
+        self.assertConvertedEqual('3com.com')
+        self.assertConvertedEqual('10.org')
+        self.assertConvertedEqual('10ab.10ab')
+        self.assertConvertedEqual('ab-c.com')
+        self.assertConvertedEqual('abc.com-org')
+        self.assertConvertedEqual('abc.0-0')
 
     def test_max_segment_size(self):
         self.assertConvertedEqual('host.%s.com' % ('x' * 63))
@@ -731,6 +856,35 @@ class URITypeTests(TypeTestHelper, unittest.TestCase):
         self.assertConvertedValue('http://www.example.com',
                                   'http://www.example.com')
 
+    def test_equality(self):
+        a = types.URI()
+        b = types.URI()
+        self.assertEqual(a, b)
+
+    def test_equality_length(self):
+        a = types.URI(max_length=5)
+        b = types.URI(max_length=5)
+        self.assertEqual(a, b)
+
+    def test_equality_length_not(self):
+        a = types.URI()
+        b = types.URI(max_length=5)
+        c = types.URI(max_length=10)
+        self.assertNotEqual(a, b)
+        self.assertNotEqual(c, b)
+
+    def test_equality_schemes(self):
+        a = types.URI(schemes=['ftp'])
+        b = types.URI(schemes=['ftp'])
+        self.assertEqual(a, b)
+
+    def test_equality_schemes_not(self):
+        a = types.URI()
+        b = types.URI(schemes=['ftp'])
+        c = types.URI(schemes=['http'])
+        self.assertNotEqual(a, b)
+        self.assertNotEqual(c, b)
+
 
 class PortTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Port()
@@ -762,14 +916,31 @@ class PortTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.Port(choices=[80, 457])
         self.assertEqual('Port(choices=[80, 457])', repr(t))
 
-    def test_choices(self):
-        t = types.Port(choices=[80, 457])
+    def test_repr_with_choices_tuple(self):
+        t = types.Port(choices=(80, 457))
+        self.assertEqual('Port(choices=[80, 457])', repr(t))
+
+    def _test_with_choices(self, t):
         self.assertRaises(ValueError, t, 1)
         self.assertRaises(ValueError, t, 200)
+        self.assertRaises(ValueError, t, -457)
         t(80)
         t(457)
 
+    def test_with_choices_list(self):
+        t = types.Port(choices=[80, 457])
+        self._test_with_choices(t)
+
+    def test_with_choices_tuple(self):
+        t = types.Port(choices=(80, 457))
+        self._test_with_choices(t)
+
+    def test_with_choices_dict(self):
+        t = types.Port(choices=[(80, 'ab'), (457, 'xy')])
+        self._test_with_choices(t)
+
     def test_invalid_choices(self):
+        """Check for choices that are specifically invalid for ports."""
         self.assertRaises(ValueError, types.Port, choices=[-1, 457])
         self.assertRaises(ValueError, types.Port, choices=[1, 2, 3, 65536])
 
@@ -790,7 +961,9 @@ class PortTypeTests(TypeTestHelper, unittest.TestCase):
     def test_equal_with_same_choices(self):
         t1 = types.Port(choices=[80, 457])
         t2 = types.Port(choices=[457, 80])
-        self.assertTrue(t1 == t2)
+        t3 = types.Port(choices=(457, 80))
+        t4 = types.Port(choices=[(457, 'ab'), (80, 'xy')])
+        self.assertTrue(t1 == t2 == t3 == t4)
 
     def test_not_equal(self):
         self.assertFalse(types.Port(min=123) == types.Port(min=456))
@@ -866,19 +1039,3 @@ class PortTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.Port(max=0)
         self.assertRaises(ValueError, t, 1)
         t(0)
-
-    def test_with_choices_list(self):
-        t = types.Port(choices=[80, 457])
-        self.assertRaises(ValueError, t, 1)
-        self.assertRaises(ValueError, t, 200)
-        self.assertRaises(ValueError, t, -457)
-        t(80)
-        t(457)
-
-    def test_with_choices_tuple(self):
-        t = types.Port(choices=(80, 457))
-        self.assertRaises(ValueError, t, 1)
-        self.assertRaises(ValueError, t, 200)
-        self.assertRaises(ValueError, t, -457)
-        t(80)
-        t(457)
